#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains widget to show images
"""

from __future__ import print_function, division, absolute_import

from artella.core import qtutils, resource
from artella.widgets import theme

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets, QtGui


if not qtutils.QT_AVAILABLE:
    class ArtellaImage(object):
        def __init__(self, *args, **kwargs):
            pass
else:
    class ArtellaImage(QtWidgets.QLabel, object):
        def __init__(self, parent=None):
            super(ArtellaImage, self).__init__(parent)

            self._default_pixmap = resource.pixmap('artella')
            self._pixmap = self._default_pixmap
            self._artella_size = 0
            self.set_artella_size(theme.theme().default_size)

        def set_artella_size(self, value):
            self._artella_size = value
            self._set_artella_size()

        def _set_artella_size(self):
            self.setFixedSize(QtCore.QSize(self._artella_size, self._artella_size))
            self._set_artella_image()

        def _set_artella_image(self):
            self.setPixmap(self._pixmap.scaledToWidth(self.height(), QtCore.Qt.SmoothTransformation))

        def set_artella_image(self, value):
            """
            Set avatar image.
            :param value: QPixmap or None.
            :return: None
            """
            if value is None:
                self._pixmap = self._default_pixmap
            elif isinstance(value, QtGui.QPixmap):
                self._pixmap = value
            else:
                raise TypeError("Input argument 'value' should be QPixmap or None, but get {}".format(type(value)))
            self._set_artella_image()

        def get_artella_image(self):
            return self._pixmap

        def get_artella_size(self):
            return self._artella_size

        artella_image = QtCore.Property(QtGui.QPixmap, get_artella_image, set_artella_image)
        artella_size = QtCore.Property(int, get_artella_size, set_artella_size)

        @classmethod
        def tiny(cls, image=None):
            inst = cls()
            inst.set_artella_size(theme.theme().tiny)
            inst.set_artella_image(image)
            return inst

        @classmethod
        def small(cls, image=None):
            inst = cls()
            inst.set_artella_size(theme.theme().small)
            inst.set_artella_image(image)
            return inst

        @classmethod
        def medium(cls, image=None):
            inst = cls()
            inst.set_artella_size(theme.theme().medium)
            inst.set_artella_image(image)
            return inst

        @classmethod
        def large(cls, image=None):
            inst = cls()
            inst.set_artella_size(theme.theme().large)
            inst.set_artella_image(image)
            return inst

        @classmethod
        def huge(cls, image=None):
            inst = cls()
            inst.set_artella_size(theme.theme().huge)
            inst.set_artella_image(image)
            return inst



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella API implementation
"""

from __future__ import print_function, division, absolute_import

from artella.core import dccplugin


def ping():
    """
    Test call that returns whether the Artella Drive is valid or not

    :return: Returns a success response or auth failure message
    :rtype: dict
    :example:
    >>> self.ping()
    {
        message: 'OK';
        response: true;
        status_code: 200;
    }
    """

    client = get_client()
    if not client:
        return

    return client.ping()


def get_local_root():
    """
    Returns the local storage root path for this machine by asking to remote server.
    .. note::
        If local root cannot be retrieved from server (this can happen because server is down or because user
        doesn't have a working internet connection) then this function will try to retrieve the local root path
        from ARTELLA_FOLDER_ROOT environment variable. This environment variable must be defined by users manually
        to point to local root path.

    :return: Absolute path where files are stored locally
    :rtype: str or None
    :example:
    >>> self.get_local_root()
    "C:/Users/artella/artella-files"
    """

    client = get_client()
    if not client:
        return

    return client.get_local_root()


def get_local_projects():
    """
    Returns all available project files found in the local user machine

    :return: List of dictionaries containing all the information of the current available
    Artella on local user machine projects
    :rtype: dict(str, dict(str, str))
    """

    client = get_client()
    if not client:
        return

    return client.get_local_projects()


def update_paths(file_path=None, show_dialogs=True, call_post_function=True, skip_save=True):
    """
    Updates all file paths of the given file path to make sure that they point to valid Artella file paths

    :param  str or list(str) file_path:
    :param bool show_dialogs:
    :param bool call_post_function:
    :param bool skip_save:
    :return:
    """

    return dccplugin.DccPlugin().update_paths(
        file_path=file_path, show_dialogs=show_dialogs, call_post_function=call_post_function, skip_save=skip_save)


def relative_path_to_absolute_path(relative_path, project_id=None):
    """
    Converts a relative path to an absolute local file path taking into account the given project id

    :param str relative_path: Relative path to a file
    :param str or None project_id: project id the relative path file belongs to
    :return: Absolute local file path of the relative path in the given project
    :rtype: str
    """

    client = get_client()
    if not client:
        return

    return client.relative_path_to_absolute_path(relative_path, project_id=project_id)


def translate_path(file_path):
    """
    Converts a file path to a local file path taking into account the available projects.

    :param str file_path: File path we want to to translate to its user local version
    :return: User local version of the given path
    :rtype: str
    :example:
    >>> self.translate_path("C:/Users/Bobby/artella-files/ProjectA/Assets/Characters/A/Model/a.ma")
    "C:/Users/Tomi/artella/data/ProjectA/Assets/Characters/A/Model/a.ma"
    >>> self.translate_path("/ProjectA/Assets/Characters/A/Model/a.ma")
    "C:/Users/Tomi/artella/data/ProjectA/Assets/Characters/A/Model/a.ma"
    """

    return dccplugin.DccPlugin().translate_path(file_path)


def is_artella_path(file_path=None):
    """
    Returns whether or not given file path is an Artella file path or not
    A path is considered to be an Artella path if the path is located inside the Artella project folder
    in the user machine

    :param str file_path: path to check. If not given, current DCC scene file path will be used
    :return: True if the given file path is an Artella path; False otherwise.
    :rtype: bool
    """

    return dccplugin.DccPlugin().is_artella_path(file_path)


def is_path_translated(file_path):
    """
    Returns whether or not given path is already translated to a valid Artella path

    :param file_path: str, path you want to check translation validation
    :return: True if the given path is an already translated Artella path; False otherwise
    :rtype: bool
    :example:
    >>> self.is_path_translated("C:/Users/Bobby/artella-files/ProjectA/Assets/Characters/A/Model/a.ma")
    False
    >>> self.is_path_translated("$ART_LOCAL_ROOT/ProjectA/refs/ref.png")
    True
    """

    return dccplugin.DccPlugin().is_path_translated(file_path)


def convert_path(file_path):
    """
    Converts given path to a path that Artella can understand

    :param str file_path: File path we want to convert
    :return: str
    :rtype: str
    :example:
    >>> self.translate_path("C:/Users/Bobby/artella-files/ProjectA/Assets/Characters/A/Model/a.ma")
    "$$ART_LOCAL_ROOT/ProjectA/Assets/Characters/A/Model/a.ma"
    >>> self.translate_path("/ProjectA/Assets/Characters/A/Model/a.ma")
    "$ART_LOCAL_ROOT/ProjectA/Assets/Characters/A/Model/a.ma"
    """

    return dccplugin.DccPlugin().convert_path(file_path)


def download_file(file_path, show_dialogs=True):
    """
    Downloads a file from Artella server

    :param str file_path: File we want to download from Artella server
    :param bool show_dialogs: Whether UI dialogs should appear or not.
    :return: True if the download operation was successful; False otherwise.
    :example:
    >>> self.download_file("C:/Users/artella/artella-files/ProjectA/Assets/test/model/hello.a")
    True
    """

    return dccplugin.DccPlugin().download_file(file_path=file_path, show_dialogs=show_dialogs)


def make_new_version(file_path=None, comment=None, do_lock=False):
    """
    Uploads a new file/folder or a new version of current opened DCC scene file

    :param str file_path: Optional path of the file we want to create new version of. If not given, current
        opened DCC scene file path will be used.
    :param str comment: Optional comment to add to new version metadata. If not given, a generic message will be
        used.
    :param bool do_lock: Whether or not to force the lock of the file to make a new version. With new Artella
        version this is not mandatory.
    :return: True if the make new version operation is completed successfully; False otherwise.
    :rtype: bool
    """

    return dccplugin.DccPlugin().make_new_version(file_path=file_path, comment=comment, do_lock=do_lock)


def file_current_version(file_path):
    """
    Returns current version of the given file

    :param str file_path: Absolute local file path to retrieve current local version of
    :param str _status: new file status
    :return: Current local version of the given file path
    :rtype: int
    """

    client = get_client()
    if not client:
        return

    return client.file_current_version(file_path)


def file_is_latest_version(file_path):
    """
    Returns whether or not given local file path is updated to the latest version available in Artella server.

    :param str file_path: Absolute local file path or URI path we want to check version of
    :return: True if the file is updated to the latest version available; False otherwise.
    :rtype: bool
    """

    client = get_client()
    if not client:
        return

    return client.file_is_latest_version(file_path)


def file_status(file_paths, include_remote=False):
    """
    Returns the status of a file from the remote server

    :param str or list(str) file_paths: Local path(s) or Resolved URI path(s) of a folder/file or list of
        folders/files
    :param bool include_remote: Whether to include remote server information into the response. Take into account
        that this call can slow down the server response.
    :return:
    :rtype: dict
    :example:
    >>> self.status('artella:project__fo0pohsa2sea4wyr5zmzcwnzse/refs/ref.png')
    >>> self.status('C:/Users/artella/artella-files/projects/refs/ref.png')
    {
        'project__fo0pohsa2sea4wyr5zmzcwnzse/refs/ref.png':
        {
            'local_info':
            {
                'content_length': 390223,
                'remote_version': 1,
                'name': u'Area.png',
                'modified_time': '2020-03-11T23:38:01.3985577+01:00',
                'mode': u'0666',
                'signature':
                'sha1:f67817bdfa6a828ab3a80110d2e52ca8a430afb7',
                'path': u'C:\\Users\\artella\\artella-files\\project\\refs\\ref.png'
            }
        }
    }
    """

    client = get_client()
    if not client:
        return

    return client.status(file_paths, include_remote=include_remote)


def show_success_message(text, title='', duration=None, closable=True):
    """
    Shows an info message

    :param text: str, success text to show
    :param title: str, title of the success message
    :param duration: float or None, if given, message only will appear the specified seconds
    :param closable: bool, Whether the message can be closed by the user or not (when duration is given)
    """

    return dccplugin.DccPlugin().show_success_message(text=text, title=title, duration=duration, closable=closable)


def show_info_message(text, title='', duration=None, closable=True):
    """
    Shows an info message

    :param text: str, info text to show
    :param title: str, title of the info message
    :param duration: float or None, if given, message only will appear the specified seconds
    :param closable: bool, Whether the message can be closed by the user or not (when duration is given)
    """

    return dccplugin.DccPlugin().show_info_message(text=text, title=title, duration=duration, closable=closable)


def show_warning_message(text, title='', duration=None, closable=True):
    """
    Shows a warning message

    :param text: str, warning text to show
    :param title: str, title of the warning message
    :param duration: float or None, if given, message only will appear the specified seconds
    :param closable: bool, Whether the message can be closed by the user or not (when duration is given)
    """

    return dccplugin.DccPlugin().show_warning_message(text=text, title=title, duration=duration, closable=closable)


def show_error_message(text, title='', duration=None, closable=True):
    """
    Shows an error message

    :param text: str, error text to show
    :param title: str, title of the error message
    :param duration: float or None, if given, message only will appear the specified seconds
    :param closable: bool, Whether the message can be closed by the user or not (when duration is given)
    :return:
    """

    return dccplugin.DccPlugin().show_error_message(text=text, title=title, duration=duration, closable=closable)


def is_client_available(update=False):
    """
    Returns whether or not current client is available

    :param update: bool, Whether or not remote sessions should be updated
        :return: True if the client is available and running; False otherwise.
    :rtype: ArtellaDriveClient
    """

    artella_drive_client = dccplugin.DccPlugin().get_client()
    if not artella_drive_client or not artella_drive_client.check(update=update):
        return False

    return True


def get_client(check=False, update=False):
    """
    Returns current Artella Drive Client being used by Artella DCC plugin

    :param check: bool, Whether check or not client is available and running
    :param update: bool, Whether or not remote sessions should be updated
    :return: Instance of current Artella Drive Client being used
    :rtype: ArtellaDriveClient
    """

    artella_drive_client = dccplugin.DccPlugin().get_client()
    if check and not artella_drive_client.check(update=update):
        return None
    if not artella_drive_client:
        return None

    return artella_drive_client


if __name__ == '__main__':
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Initialization module for artella
"""

from __future__ import print_function, division, absolute_import

import os
import sys
import logging.config

from artella import dcc
from artella.core import consts, utils, client, resource, plugins, dccplugin, dcc as dcc_core


def create_logger():
    """
    Creates Artella logger based on logging.ini configuration file
    :return:
    """

    logger_path = os.path.normpath(os.path.join(os.path.expanduser('~'), 'artella', 'logs'))
    if not os.path.isdir(logger_path):
        os.makedirs(logger_path)

    logging.config.fileConfig(
        os.path.normpath(os.path.join(os.path.dirname(__file__), 'logging.ini')), disable_existing_loggers=False)


def register_dcc_paths(dcc_paths=None):

    # Register DCC paths
    dccs_path = dcc_paths or list()
    valid_dcc_paths = list()
    dcc_paths_str = ''
    default_dccs_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'dccs')
    dccs_path.append(default_dccs_path)
    for dcc_path in dccs_path:
        if os.path.isdir(dcc_path):
            if dcc_path not in sys.path:
                sys.path.append(dcc_path)
            valid_dcc_paths.append(dcc_path)
    if valid_dcc_paths:
        if os.environ.get(consts.AED, None):
            env = os.environ[consts.AED].split(';')
            env.extend(valid_dcc_paths)
            clean_env = list(set([utils.clean_path(pth) for pth in env]))
            dcc_paths_str = ';'.join(clean_env)
        else:
            dcc_paths_str = ';'.join(valid_dcc_paths)
    if os.environ.get(consts.AED, ''):
        if dcc_paths_str:
            os.environ[consts.AED] += ';{}'.format(dcc_paths_str)
    else:
        os.environ[consts.AED] = dcc_paths_str


def init(
        dcc_paths=None, init_client=True, plugin_paths=None, extensions=None, dev=False,
        load_plugins=True, create_menu=True, create_callbacks=True):
    """
    Initializes Artella Plugin

    :param bool init_client: Whether or not Artella Drive Client should be initialized during initialization.
        Useful to avoid to connect to Artella client when developing DCC specific functionality.
    :param list(str) plugin_paths: List of paths where Artella plugins can be located
    :param list(str) extensions: List of extensions to register
    :param bool dev: Whether or not initialization should be done in dev mode
    :param bool load_plugins: Whether or not Artella Plugins should be loaded
    :param bool create_menu: Whether or not Artella menu should be created
    :param bool create_callbacks: Whether or not Artella DCC plugin callbacks should be created
    :return: True if Artella initialization was successful; False otherwise.
    :rtype: bool
    """

    register_dcc_paths(dcc_paths)

    plugins_path = plugin_paths if plugin_paths is not None else list()
    extensions = extensions if extensions is not None else list()

    artella_logger = logging.getLogger('artella')

    # Make sure that Artella Drive client and DCC are cached during initialization
    current_dcc = dcc_core.current_dcc()
    if not current_dcc:
        artella_logger.error('Impossible to load Artella Plugin because no DCC is available!')
        return False

    shutdown(dev=dev)

    # Specific DCC extensions are managed by the client
    dcc_extensions = dcc.extensions() or list()
    extensions.extend(dcc_extensions)

    # Initialize resources and theme
    resource.register_resources_path(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'resources'))

    # Create Artella Drive Client
    artella_drive_client = client.ArtellaDriveClient.get(extensions=extensions) if init_client else None

    # Load Plugins
    if load_plugins:
        default_plugins_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'plugins')
        if default_plugins_path not in plugins_path:
            plugins_path.append(default_plugins_path)
        plugins.register_paths(plugin_paths)
        plugins.load_registered_plugins(dev=dev)

    # Initialize Artella DCC plugin
    dccplugin.DccPlugin(artella_drive_client).init(
        dev=dev, show_dialogs=False, create_menu=create_menu, create_callbacks=create_callbacks,
        init_client=init_client)

    if not dev:
        updater_plugin = plugins.get_plugin_by_id('artella-plugins-updater')
        if updater_plugin and updater_plugin.update_is_available(show_dialogs=False):
            updater_plugin.check_for_updates(show_dialogs=False)

    return True


def shutdown(dev=False):
    """
    Shutdown Artella Plugin

    :return: True if Artella shutdown was successful; False otherwise.
    :rtype: bool
    """

    try:
        plugins.shutdown(dev=dev)
        dccplugin.DccPlugin().shutdown(dev=dev)
    except Exception as exc:
        pass

    return True


create_logger()



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains implementation for Artella divider widget
"""

from __future__ import print_function, division, absolute_import

from artella.core import qtutils
from artella.widgets import label

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets


if not qtutils.QT_AVAILABLE:
    class ArtellaDivider(object):
        def __init__(self, *args, **kwargs):
            super(ArtellaDivider, self).__init__()
else:
    class ArtellaDivider(QtWidgets.QWidget, object):
        ALIGNMENT = {
            QtCore.Qt.AlignCenter: 50,
            QtCore.Qt.AlignLeft: 20,
            QtCore.Qt.AlignRight: 80,
        }

        def __init__(self, text='', orientation=QtCore.Qt.Horizontal, alignment=QtCore.Qt.AlignCenter, parent=None):
            super(ArtellaDivider, self).__init__(parent)
            self._orient = orientation
            self._text_label = label.ArtellaLabel().secondary()
            self._left_frame = QtWidgets.QFrame()
            self._right_frame = QtWidgets.QFrame()
            self._main_lay = QtWidgets.QHBoxLayout()
            self._main_lay.setContentsMargins(0, 0, 0, 0)
            self._main_lay.setSpacing(0)
            self._main_lay.addWidget(self._left_frame)
            self._main_lay.addWidget(self._text_label)
            self._main_lay.addWidget(self._right_frame)
            self.setLayout(self._main_lay)

            if orientation == QtCore.Qt.Horizontal:
                self._left_frame.setFrameShape(QtWidgets.QFrame.HLine)
                self._left_frame.setFrameShadow(QtWidgets.QFrame.Sunken)
                self._right_frame.setFrameShape(QtWidgets.QFrame.HLine)
                self._right_frame.setFrameShadow(QtWidgets.QFrame.Sunken)
            else:
                self._text_label.setVisible(False)
                self._right_frame.setVisible(False)
                self._left_frame.setFrameShape(QtWidgets.QFrame.VLine)
                self._left_frame.setFrameShadow(QtWidgets.QFrame.Plain)
                self.setFixedWidth(2)
            self._main_lay.setStretchFactor(self._left_frame, self.ALIGNMENT.get(alignment, 50))
            self._main_lay.setStretchFactor(self._right_frame, 100 - self.ALIGNMENT.get(alignment, 50))
            self._text = None
            self.set_artella_text(text)

        def set_artella_text(self, value):
            self._text = value
            self._text_label.setText(value)
            if self._orient == QtCore.Qt.Horizontal:
                self._text_label.setVisible(bool(value))
                self._right_frame.setVisible(bool(value))

        def get_artella_text(self):
            return self._text

        artella_text = QtCore.Property(basestring, get_artella_text, set_artella_text)

        @classmethod
        def left(cls, text=''):
            return cls(text, alignment=QtCore.Qt.AlignLeft)

        @classmethod
        def right(cls, text=''):
            return cls(text, alignment=QtCore.Qt.AlignRight)

        @classmethod
        def center(cls, text=''):
            return cls(text, alignment=QtCore.Qt.AlignCenter)

        @classmethod
        def vertical(cls):
            return cls(orientation=QtCore.Qt.Vertical)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains core callback functionality
"""

from __future__ import print_function, division, absolute_import

import inspect
import logging
import traceback

from artella import dcc
from artella.core.dcc import callback
from artella.core import dcc as dcc_core

# Cache used to store Artella callbacks
ARTELLA_CALLBACKS_CACHE = dict()

logger = logging.getLogger('artella')


def initialize_callbacks():

    global ARTELLA_CALLBACKS_CACHE
    if ARTELLA_CALLBACKS_CACHE:
        return

    shutdown_type = None
    if hasattr(callback.Callbacks(), 'ShutdownCallback'):
        shutdown_type = getattr(callback.Callbacks(), 'ShutdownCallback')

    for callback_name in dcc_core.callbacks():
        callback_type = getattr(dcc_core.DccCallbacks, callback_name)[1]['type']
        if callback_type == 'simple':
            callback_type = SimpleCallbackWrapper
        elif callback_type == 'filter':
            callback_type = FilterCallbackWrapper
        else:
            callback_type = SimpleCallbackWrapper

        callback_class = getattr(callback.Callbacks(), '{}Callback'.format(callback_name), None)
        if not callback_class:
            logger.warning(
                'Dcc {} does not provides a Callback implementation for {}Callback. Skipping ...'.format(
                    dcc.name(), callback_name))
            continue

        # This should not be necessary. We added just to be sure that no callbacks are registered in scenes
        # because that could break user DCC session.
        new_callback = ARTELLA_CALLBACKS_CACHE.get(callback_name, None)
        if new_callback:
            new_callback.cleanup()

        ARTELLA_CALLBACKS_CACHE[callback_name] = callback_type(callback_class, shutdown_type)

        logger.debug('Creating Callback: {} | {}'.format(callback_name, callback_class))


def register(callback_type, fn):
    """
    Register a new callback if given callback_type exists in current DCC
    :param str callback_type: type of callback
    :param fn: Python function that will be called when callback is emitted
    """

    global ARTELLA_CALLBACKS_CACHE
    if not ARTELLA_CALLBACKS_CACHE:
        return

    if isinstance(callback_type, (list, tuple)):
        callback_type = callback_type[0]

    if inspect.isclass(callback_type):
        callback_type = callback_type.__name__

    if callback_type.endswith('Callback'):
        callback_type = callback_type.replace('Callback', '')

    if callback_type in ARTELLA_CALLBACKS_CACHE.keys():
        ARTELLA_CALLBACKS_CACHE[callback_type].register(fn)


def unregister(callback_type, fn):
    """
    Unregisters given function from given callback type
    :param str callback_type: type of callback
    :param fn: Python function we want to unregister from callback
    """

    if isinstance(callback_type, (list, tuple)):
        callback_type = callback_type[0]

    if callback_type in ARTELLA_CALLBACKS_CACHE.keys():
        ARTELLA_CALLBACKS_CACHE[callback_type].unregister(fn)


def uninitialize_callbacks():

    global ARTELLA_CALLBACKS_CACHE
    if not ARTELLA_CALLBACKS_CACHE:
        return

    for callback_name, callback_obj in ARTELLA_CALLBACKS_CACHE.items():
        callback_obj.cleanup()

    ARTELLA_CALLBACKS_CACHE.clear()


class CallbackWrapper(object):
    """
    Class that wraps DCC callbacks
    """

    def __init__(self, notifier, shutdown_notifier):
        super(CallbackWrapper, self).__init__()

        self._notifier = notifier
        self._enabled_stack = list()
        self._registry = list()
        self._shutdown_notifier = None
        self._shutdown_token = None
        if shutdown_notifier and notifier != shutdown_notifier:
            self._shutdown_notifier = shutdown_notifier
            self._shutdown_token = self._shutdown_notifier.register(self._shutdown)

    def __del__(self):
        if self._notifier:
            self._shutdown([])

    @property
    def valid(self):
        """
        Property to query the validity of this callback

        :return: True if the callback has a notifier; False otherwise
        """

        return bool(self._notifier)

    @property
    def empty(self):
        """
        Property to query the existence of listeners registered to this callback

        :return: True if the callback has listener registered; False otherwise
        """

        return True

    @property
    def connected(self):
        """
        Property to query the 'connected' state of the callback.

        :return: True if the callback has connected itself with the INotifier implementation.
        """

        return False

    @property
    def enabled(self):
        """
        Property to query the 'not empty' and 'connected' state of the callback.

        :return: True if the callback has listeners and is connected the to the INotifier implementation.
        """

        return False

    @property
    def registry(self):
        """
        Property to query all registered functions of a callback

        :return: list<fn>
        """

        return None

    @enabled.setter
    def enabled(self, value):
        """
        Property to set the 'enable' state of the callback.  Modifying the enable state either toggles
        the 'connected' state of the callback but maintains the list of listeners.

        :param bool value: value The enable state of the callback.
        """
        pass

    def resume(self):
        """
        Function that resume the notification connection

        :return: True if the callback connection has been resumed properly; False otherwise
        """

        return self._pop()

    def suspend(self):
        """
        Suspends callback connection
        """

        self._push(False)

    def register(self, fn):
        """
        Adds a listener to this instance.

        :param fn: A valid python function with a variable number of arguments (i.e. *args).
        """

        pass

    def unregister(self, fn):
        """
        Removes a listener from this instance.

        :param fn: A valid python function with a variable number of arguments (i.e. *args).
        """

        pass

    def cleanup(self):
        """
        Function to terminate the callback
        """

        return self._shutdown()

    def _connect(self, fn):
        """
        Internal callback registration function

        :param fn: Python function to register as a listener in the sender
        """

        return self._notifier.register(fn)

    def _disconnect(self, token):
        """
        Internal callback unregister function

        :param token: valid token returned from a previous _connect call
        :return: None if registration was not done or the unchanged value from token otherwise
        """

        if token:
            return self._notifier.unregister(token)

        return None

    def _filter(self, *args):
        """
        Internal function to evaluate if the callback from the notifier is valid.
        Tests the validity of the message with the custom function.

        :param args:  Variable list of arguments received from the notifier
        :return: A tuple of indeterminant length and type if callbacks should be passed to listener; (False, None)
            otherwise.
        """

        return self._notifier.filter(*args)

    def _push(self, state):
        """
        Internal function to set the enable state while maintaining a history of previous enabled states

        :param bool state: the enable state of the callback
        """

        if self.valid:
            self._enabled_stack.append(self.enabled)
            self.enabled = state

    def _pop(self):
        """
        Internal function to restore the enable state to a previous enabled state

        :return: True if the callback is enabled as a result of the registration or False otherwise
        """

        if self.valid and self._enabled_stack:
            self.enabled = self._enabled_stack.pop()

        return self.enabled

    def _shutdown(self, *args):
        """
        Internal force that forces the unregistering and disconnection from sender.

        :param args:
        :return:
        """

        if self._shutdown_token and self._shutdown_notifier:
            self._shutdown_notifier.unregister(self._shutdown_token)
        self._shutdown_notifier = None
        self._notifier = None


class SimpleCallbackWrapper(CallbackWrapper, object):
    """
    Simple implementation for Callback wrapper
    It maintains a one-to-one relation between listener and notifiers without any event filtering
    """

    class RegistryEntry(object):
        def __init__(self, callback, token):
            self.callback = callback
            self.token = token

    def __init__(self, notifier, shutdown_notifier):
        super(SimpleCallbackWrapper, self).__init__(notifier=notifier, shutdown_notifier=shutdown_notifier)

    @property
    def empty(self):
        """
        Property to query the existence of listeners to this callback.

        :return: True if the callback has listeners registered; False otherwise.
        """

        return not bool(self._registry)

    @property
    def connected(self):
        """
        Property to query the 'connected' state of the callback.

        :return: True if the callback has connected itself with the INotifier implementation.
        """

        return all(e.token for e in self._registry)

    @property
    def enabled(self):
        """
        Property to query the 'not empty' and 'connected' state of the callback.

        :return: True if the callback has listeners and is connected the to the INotifier implementation.
        """

        return not self.empty and bool(self.connected)

    @enabled.setter
    def enabled(self, value):
        """!
        Property to set the 'enable' state of the callback.  Modifying the enable state either toggles
        the 'connected' state of the callback but maintains the list of listeners.

        :param bool value: The enable state of the callback.
        """

        for entry in self._registry:
            if not value and entry.token:
                entry.token = self._disconnect(entry.token)
            elif value and not entry.token:
                entry.token = self._connect(entry.callback)

    def register(self, fn):
        """
        Adds a listener to this instance

        :param fn fn: a valid Python function with a variable number of arguments (exp. *args)
        """

        entry = next((e for e in self._registry if e.callback == fn), None)
        logger.debug(
            'Started: ({}) {} Register - fn:"{}", entry:"{}"'.format(
                str(self._notifier), self.__class__.__name__, str(fn), str(entry)))
        if not entry:
            token = self._connect(fn) if self.connected else None
            logger.debug(
                '({}) {} Register - token:"{}"'.format(str(self._notifier), self.__class__.__name__, str(token)))
            self._registry.append(SimpleCallbackWrapper.RegistryEntry(fn, token))
        logger.debug('Completed: ({}) {} Register'.format(str(self._notifier), self.__class__.__name__))

    def unregister(self, fn):
        """
        Removes a listener from this instance

        :param fn: a valid Python function with a variable number of arguments (exp. *args)
        """

        entry = next((e for e in self._registry if e.callback == fn), None)
        logger.debug(
            'Started: ({}) {} Unregister - fn:"{}", entry:"{}"'.format(
                str(self._notifier), self.__class__.__name__, str(fn), str(entry)))
        if entry:
            self._disconnect(entry.token)
            self._registry.remove(entry)
        logger.debug('Completed: ({}) {} Unregister'.format(str(self._notifier), self.__class__.__name__))

    def _shutdown(self, *args):
        """
        Forces an unregistering from the notifier
        """

        logger.debug('Started: ({}) {} Shutdown'.format(str(self._notifier), self.__class__.__name__))
        for entry in self._registry:
            logger.debug(
                '{}._shutdown - Disconnecting ({}) {}'.format(str(self._notifier), self.__class__.__name__, str(entry)))
            self._disconnect(entry.token)
        del self._registry[:]

        super(SimpleCallbackWrapper, self)._shutdown(*args)
        logger.debug('Complete: ({}) {} Shutdown'.format(str(self._notifier), self.__class__.__name__))


class FilterCallbackWrapper(CallbackWrapper, object):
    """
    Filter implementation for Callback wrapper, It allows to filtering the callback generated from the notifier.
    It maintains a many-to-one relation between listeners and notifiers.
    """

    class RegistryEntry(object):
        def __init__(self, callback):
            self.callback = callback

    def __init__(self, notifier, shutdown_notifier):
        super(FilterCallbackWrapper, self).__init__(notifier=notifier, shutdown_notifier=shutdown_notifier)

        self._token = None

    @property
    def empty(self):
        """
        Property to query the existence of listeners to this callback.

        :return: True if the callback has listeners registered; False otherwise.
        """

        return not bool(self._registry)

    @property
    def connected(self):
        """
        Property to query the 'connected' state of the callback.

        :return: True if the callback has connected itself with the INotifier implementation.
        """

        return bool(self._token)

    @property
    def enabled(self):
        """
        Property to query the 'not empty' and 'connected' state of the callback.

        :return: True if the callback has listeners and is connected the to the INotifier implementation.
        """

        return not self.empty and bool(self.connected)

    @enabled.setter
    def enabled(self, value):
        """
        Property to set the 'enable' state of the callback.  Modifying the enable state either toggles
        the 'connected' state of the callback but maintains the list of listeners.

        :param value: The enable state of the callback.
        """

        if not value and self._token:
            self._token = self._disconnect(self._token)
        elif value and not self._token:
            self._token = self._connect(self._notify)

    def _shutdown(self, *args):
        """
        Forces an unregistering from the notifier
        """

        logger.debug('Started: ({}) {} Shutdown'.format(str(self._notifier), self.__class__.__name__))
        if self._token:
            self._token = self._disconnect(self._token)
            self._token = None
        del self._registry[:]

        super(self.__class__, self)._shutdown(*args)
        logger.debug('Complete: ({}) {} Shutdown'.format(str(self._notifier), self.__class__.__name__))

    def _notify(self, *args):
        """
        Internal function registered with the notifier. Evaluates the condition with _filter during the callback.
        If its valid, it will broadcast the callback to the listener via _execute().
        All notifier data is passed on to the user via _execute().

        :param args: A variable list of arguments received from the notifier
        :return:
        """

        fn_args = self._filter(*args)
        if fn_args[0]:
            self._execute(*fn_args[1:])

    def _execute(self, *args):
        """
        Internal function to notify all listeners registered to the current instance of the class

        :param args: A variable list of arguments received from the notifier
        """

        for entry in self._registry:
            try:
                entry.callback(*args)
            except Exception as exc:
                logger.error('{} | {}'.format(exc, traceback.format_exc()))

    def register(self, fn):
        """
        Adds a listener to this instance

        :param fn: a valid Python function with a variable number of arguments (exp. *args)
        """

        logger.debug(
            'Started: ({}) {} Register - fn:"{}", IsEmpty:"{}"'.format(
                str(self._notifier), self.__class__.__name__, str(fn), bool(self.empty)))
        if self.empty:
            self._token = self._connect(self._notify)
            logger.debug(
                '({}) {} Register - token:"{}"'.format(str(self._notifier), self.__class__.__name__, str(self._token)))
        self._registry.append(FilterCallbackWrapper.RegistryEntry(fn))
        logger.debug('Completed: ({}) {} Register'.format(str(self._notifier), self.__class__.__name__))

    def unregister(self, fn):
        """
        Removes a listener from this instance

        :param fn: a valid Python function with a variable number of arguments (exp. *args)
        """

        entry = next((e for e in self._registry if e.callback == fn), None)
        logger.debug(
            'Started: ({}) {} Unregister - fn:"{}", IsEmpty:"{}"'.format(
                str(self._notifier), self.__class__.__name__, str(fn), bool(self.empty)))

        if entry:
            self._registry.remove(entry)

        if self.empty and self.connected:
            logger.debug(
                '({}) {} Unregister token:"{}"'.format(str(self._notifier), self.__class__.__name__, str(self._token)))
            self._token = self._disconnect(self._token)
        logger.debug('Completed: ({}) {} Unregister'.format(str(self._notifier), self.__class__.__name__))



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC abstract callback implementation
"""

from artella import dcc
from artella.core.dcc import reroute
from artella.core.utils import abstract, add_metaclass


class _MetaCallbacks(type):

    def __call__(cls, *args, **kwargs):
        if dcc.is_maya():
            from artella.dccs.maya import callback as maya_callback
            return maya_callback.Callbacks
        else:
            return AbstractCallback


class AbstractCallback(object):
    """
    Class that defines basic callback abstract functions
    """

    @classmethod
    @abstract
    @reroute
    def filter(cls, *args):
        """
        Used to process function callback arguments during the execution of a callback
        :param args: List of arguments pass from the callback function to be evaluated
        :return: Return tuple of flags
        :rtype: tuple(bool, bool)
        """

        pass

    @classmethod
    @abstract
    @reroute
    def register(cls, fn):
        """
        Registers given Python function as callback

        :param fn: function, Python function to register
        :return: token that can be used later to unregister the callback function
        :rtype: str
        """

        pass

    @classmethod
    @abstract
    @reroute
    def unregister(cls, token):
        """
        Unregisters Python function callback linked to given token
        :param token: Token provided by register function to unregister a specific callback function
        """

        pass


class AbstractCallbacks(object):
    """
    Class that contains all supported callback definitions
    """

    class ShutdownCallback(AbstractCallback, object):
        """
        Callback that is called before DCC app is closed
        """

        pass

    class BeforeOpenCheckCallback(AbstractCallback, object):
        """
        Callback that is called before a file is opened
        """

        pass

    class AfterOpenCallback(AbstractCallback, object):
        """
        Callback that is called before opening a DCC scene
        """

        pass

    class SceneBeforeSaveCallback(AbstractCallback, object):
        """
        Callback that is called before a DCC scene is saved
        """

        pass

    class SceneCreatedCallback(AbstractCallback, object):
        """
        Callback that is called when a new DCC scene is created
        """

        pass

    class AfterLoadReferenceCallback(AbstractCallback, object):
        """
        Callback that is called after a reference file is loaded
        """

        pass

    class BeforeCreateReferenceCheckCallback(AbstractCallback, object):
        """
        Callback that is called before a new reference is created
        """

        pass


@add_metaclass(_MetaCallbacks)
class Callbacks(AbstractCallbacks):
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella Snackbar widget
"""

from __future__ import print_function, division, absolute_import

from artella import dcc
from artella.core import qtutils, resource
from artella.widgets import image, label, button, divider, theme

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets


class SnackBarTypes(object):
    ARTELLA = 'artella'
    INFO = 'info'
    SUCCESS = 'success'
    WARNING = 'warning'
    ERROR = 'error'


if not qtutils.QT_AVAILABLE:
    class SnackBarMessage(object):
        def __init__(self, *args, **kwargs):
            pass
else:
    class SnackBarMessage(QtWidgets.QWidget, object):

        closed = QtCore.Signal()

        DEFAULT_DURATION = 6
        DEFAULT_TOP = 180

        def __init__(self, text='', title='', duration=None, artella_type=None, closable=False, parent=None):

            if parent is None:
                parent = dcc.get_main_window()
            current_type = artella_type or SnackBarTypes.ARTELLA

            super(SnackBarMessage, self).__init__(parent)

            self.setWindowFlags(QtCore.Qt.FramelessWindowHint | QtCore.Qt.Dialog | QtCore.Qt.WA_DeleteOnClose)
            self.setAttribute(QtCore.Qt.WA_StyledBackground)
            self.setAttribute(QtCore.Qt.WA_TranslucentBackground)

            main_layout = QtWidgets.QVBoxLayout()
            main_layout.setContentsMargins(2, 2, 2, 2)
            main_layout.setSpacing(2)
            self.setLayout(main_layout)

            main_frame = QtWidgets.QFrame()
            main_frame.setObjectName('mainFrame')
            frame_layout = QtWidgets.QVBoxLayout()
            frame_layout.setContentsMargins(5, 5, 5, 5)
            frame_layout.setSpacing(5)

            main_frame.setLayout(frame_layout)
            main_layout.addWidget(main_frame)

            info_layout = QtWidgets.QHBoxLayout()

            artella_label_layout = QtWidgets.QHBoxLayout()
            artella_label = image.ArtellaImage.small()
            artella_label.set_artella_image(resource.pixmap('artella_white'))
            self._close_btn = button.ArtellaToolButton(parent=self).image('close').tiny().icon_only()
            self._close_btn.setVisible(closable or False)
            self._close_btn.clicked.connect(self.close)
            if closable:
                artella_label_layout.addSpacing(20)
            artella_label_layout.addStretch()
            artella_label_layout.addWidget(artella_label)
            artella_label_layout.addStretch()
            artella_label_layout.addWidget(self._close_btn)
            title_layout = QtWidgets.QHBoxLayout()
            self._title_label = label.ArtellaLabel(parent=self).strong()
            self._title_label.setText(title)
            self._title_label.setVisible(bool(text))
            title_layout.addStretch()
            title_layout.addWidget(self._title_label)
            title_layout.addStretch()

            self._icon_label = image.ArtellaImage.small()
            self._icon_label.set_artella_image(
               resource.pixmap('{}'.format(current_type), color=vars(theme.theme()).get(current_type + '_color')))

            self._content_label = label.ArtellaLabel(parent=self)
            self._content_label.setText(text)
            info_layout.addStretch()
            info_layout.addWidget(self._icon_label)
            info_layout.addWidget(self._content_label)
            info_layout.addStretch()

            frame_layout.addLayout(artella_label_layout)
            frame_layout.addWidget(divider.ArtellaDivider())
            frame_layout.addLayout(title_layout)
            frame_layout.addLayout(info_layout)

            self._setup_timers(duration)

            self._on_fade_in()

        @property
        def text(self):
            return self._text

        @text.setter
        def text(self, text):
            self._text = str(text)
            self._label.setText(self._text)
            self.setVisible(bool(self._text))

        @classmethod
        def artella(cls, text, title='', parent=None, duration=None, closable=None):
            inst = cls(text, title=title, artella_type=SnackBarTypes.ARTELLA,
                       duration=duration, closable=closable, parent=parent)
            theme.theme().apply(inst)
            inst.show()

            return inst

        @classmethod
        def info(cls, text, title='', parent=None, duration=None, closable=None):
            inst = cls(text, title=title, artella_type=SnackBarTypes.INFO,
                       duration=duration, closable=closable, parent=parent)
            theme.theme().apply(inst)
            inst.show()

            return inst

        @classmethod
        def success(cls, text, title='', parent=None, duration=None, closable=None):
            inst = cls(text, title=title, artella_type=SnackBarTypes.SUCCESS,
                       duration=duration, closable=closable, parent=parent)
            theme.theme().apply(inst)
            inst.show()

            return inst

        @classmethod
        def warning(cls, text, title='', parent=None, duration=None, closable=None):
            inst = cls(text, title=title, artella_type=SnackBarTypes.WARNING,
                       duration=duration, closable=closable, parent=parent)
            theme.theme().apply(inst)
            inst.show()

            return inst

        @classmethod
        def error(cls, text, title='', parent=None, duration=None, closable=None):
            inst = cls(text, title=title, artella_type=SnackBarTypes.ERROR,
                       duration=duration, closable=closable, parent=parent)
            theme.theme().apply(inst)
            inst.show()

            return inst

        def _setup_timers(self, duration):
            close_timer = QtCore.QTimer(self)
            anim_timer = QtCore.QTimer(self)
            close_timer.setSingleShot(True)
            close_timer.timeout.connect(self.close)
            close_timer.timeout.connect(self.closed.emit)
            anim_timer.timeout.connect(self._on_fade_out)
            close_timer.setInterval((duration or self.DEFAULT_DURATION) * 1000)
            anim_timer.setInterval((duration or self.DEFAULT_DURATION) * 1000 - 300)

            close_timer.start()
            anim_timer.start()

            self._pos_anim = QtCore.QPropertyAnimation(self)
            self._pos_anim.setTargetObject(self)
            self._pos_anim.setEasingCurve(QtCore.QEasingCurve.OutCubic)
            self._pos_anim.setDuration(300)
            self._pos_anim.setPropertyName('pos')

            self._opacity_anim = QtCore.QPropertyAnimation(self)
            self._opacity_anim = QtCore.QPropertyAnimation()
            self._opacity_anim.setTargetObject(self)
            self._opacity_anim.setDuration(300)
            self._opacity_anim.setEasingCurve(QtCore.QEasingCurve.OutCubic)
            self._opacity_anim.setPropertyName('windowOpacity')
            self._opacity_anim.setStartValue(0.0)
            self._opacity_anim.setEndValue(1.0)

        def resizeEvent(self, event):
            self.updateGeometry()
            self._calculate_position()
            super(SnackBarMessage, self).resizeEvent(event)

        def _calculate_position(self, parent=None):
            """
            Internal function that calculates a proper position for the snack bar relative to its parent
            """

            parent = parent or self.parent()
            parent_geo = parent.geometry()
            pos = parent_geo.topLeft() if parent.parent() is None else parent.mapToGlobal(parent_geo.topLeft())
            offset = 0
            for child in parent.children():
                if isinstance(child, SnackBarMessage) and child.isVisible():
                    offset = max(offset, child.y() + 10 + child.geometry().height() / 2)
            base_pos = pos.y() + SnackBarMessage.DEFAULT_TOP
            target_x = pos.x() + parent_geo.width() / 2 - self.size().width() / 2
            target_y = (offset + 50) if offset else base_pos
            self._pos_anim.setStartValue(QtCore.QPoint(target_x, target_y - 40))
            self._pos_anim.setEndValue(QtCore.QPoint(target_x, target_y))

        def _on_fade_out(self):
            """
            Internal callback function that fades out snack bar widget
            """

            try:
                self._pos_anim.setDirection(QtCore.QAbstractAnimation.Backward)
                self._pos_anim.start()
                self._opacity_anim.setDirection(QtCore.QAbstractAnimation.Backward)
                self._opacity_anim.start()
            except Exception:
                pass

        def _on_fade_in(self):
            """
            Internal callback function that fades in snack bar widget
            """

            try:
                self._pos_anim.start()
                self._opacity_anim.start()
            except Exception:
                pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains download manager widget for Artella
"""

from __future__ import print_function, division, absolute_import

from artella import dcc
from artella.core import utils


class _MetaDownloader(type):
    def __call__(cls, *args, **kwargs):
        if dcc.is_maya():
            from artella.dccs.maya import downloader
            _DCC_PLUGIN = type.__call__(downloader.MayaDownloader, *args, **kwargs)
        else:
            _DCC_PLUGIN = type.__call__(BaseDownloader, *args, **kwargs)

        return _DCC_PLUGIN


class AbstractDownloader(object):

    @utils.abstract
    def download(self, file_paths, show_dialogs=True):
        pass


class BaseDownloader(AbstractDownloader):

    def download(self, file_paths, show_dialogs=True):
        pass


@utils.add_metaclass(_MetaDownloader)
class Downloader(AbstractDownloader):
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains functions to manage Artella plugins
"""

from __future__ import print_function, division, absolute_import

import os
import sys
import inspect
import logging

from artella import dcc
from artella.core import consts, utils, plugin

logger = logging.getLogger('artella')

_PLUGINS = dict()
_PLUGIN_PATHS = list()


def plugins():
    """
    Returns a dictionary containing all the info of already registered Artella plugins
    :return: Dictionary with already loaded Artella plugins information
    :rtype :dict
    """

    return _PLUGINS


def plugin_paths():
    """
    Returns a list of paths where Artella Plugin Manager will find for plugins
    :return: List of plugin directory paths
    :rtype: list(str)
    """

    return _PLUGIN_PATHS


def get_plugin_by_name(plugin_name):
    """
    Returns plugin instance by its name
    :param plugin_name: Name of the plugin to retrieve
    :return: Returns plugin instance if a plugin with the given name is loaded; None otherwise.
    :rtype: ArtellaPlugin
    """

    if not _PLUGINS:
        return None

    for plugin_id, plugin_dict in _PLUGINS.items():
        plug_name = plugin_dict['name']
        if plugin_name == plug_name:
            return plugin_dict['plugin_instance']

    return None


def get_plugin_by_id(plugin_id):
    """
    Returns plugin instance by its ID
    :param plugin_id: Name of the plugin to retrieve (for example, artella-plugin-test)
    :return: Returns plugin instance if a plugin with the given ID is loaded; None otherwise.
    :rtype: ArtellaPlugin
    """

    if not _PLUGINS:
        return None

    for plug_id, plugin_dict in _PLUGINS.items():
        if plug_id == plugin_id:
            return plugin_dict['plugin_instance']


def register_paths(plugin_paths):
    """
    Registers given path into the list of Artella plugin paths.
    :param str plugin_paths: Path that will be used by Artella Plugins Manager to search plugins into.
    """

    plugin_paths = utils.force_list(plugin_paths, remove_duplicates=True)

    for plugin_path in plugin_paths:
        plugin_path = utils.clean_path(plugin_path)
        if not plugin_path or not os.path.isdir(plugin_path) or plugin_path in _PLUGIN_PATHS:
            return

        _PLUGIN_PATHS.append(plugin_path)


def load_registered_plugins(dev=False):
    """
    Loads all the plugins found in the registered plugin paths
    :return:
    """

    plugin_paths = list(set([
        utils.clean_path(plugin_path) for plugin_path in _PLUGIN_PATHS if os.path.isdir(plugin_path)]))
    if not plugin_paths:
        logger.info('No Artella Plugins found to load!')
        return

    found_paths = dict()

    for plugin_path in plugin_paths:
        for root, dirs, files in os.walk(plugin_path):
            if consts.ARTELLA_PLUGIN_CONFIG not in files:
                continue
            clean_path = utils.clean_path(root)
            found_paths[clean_path] = os.path.join(root, consts.ARTELLA_PLUGIN_CONFIG)

    if not found_paths:
        logger.info('No plugins found in registered plugin paths: {}'.format(_PLUGIN_PATHS))
        return

    for plugin_path in plugin_paths:
        for plugin_dir in os.listdir(plugin_path):
            clean_path = utils.clean_path(os.path.join(plugin_path, plugin_dir))
            if os.path.isdir(clean_path) and clean_path not in sys.path:
                sys.path.append(clean_path)

    for plugin_path, plugin_config in found_paths.items():

        # Search sub modules paths
        sub_modules_found = list()
        for sub_module in utils.iterate_modules(plugin_path):
            file_name = os.path.splitext(os.path.basename(sub_module))[0]
            if file_name.startswith('_') or file_name.startswith('test_') or sub_module.endswith('.pyc'):
                continue
            if not sub_module or sub_module in sub_modules_found:
                continue
            sub_modules_found.append(sub_module)
        if not sub_modules_found:
            continue

        # Find specific DCC plugin implementation
        dcc_name = dcc.name()
        sub_module = sub_modules_found[0]

        max_length = 50
        index = 0
        artella_module_parts = list()
        temp_sub_module = sub_module
        while True:
            if index > max_length:
                artella_module_parts = list()
                break
            base_name = os.path.basename(temp_sub_module)
            if not base_name or base_name == 'artella':
                artella_module_parts.append(base_name)
                break
            artella_module_parts.append(base_name)
            temp_sub_module = os.path.dirname(temp_sub_module)
            index += 1
        if not artella_module_parts:
            module_path = utils.convert_module_path_to_dotted_path(os.path.normpath(sub_module))
        else:
            module_path = os.path.splitext('.'.join(reversed(artella_module_parts)))[0]

        module_path_split = module_path.split('.')
        dcc_module_path = '{}.{}.{}'.format('.'.join(module_path_split[:-1]), dcc_name, module_path_split[-1])
        sub_module_obj = utils.import_module(dcc_module_path, skip_exceptions=True)
        if not sub_module_obj:
            sub_module_obj = utils.import_module(module_path)
            if not sub_module_obj:
                logger.error('Error while importing Artella Plugin module: {}'.format(module_path))
                continue

        if dev:
            plugin_version = 'DEV'
        else:
            plugin_version = None
            module_path_dir = module_path.rsplit('.', 1)[0]
            version_module_path = '{}.__version__'.format(module_path_dir)
            version_module_path = version_module_path.replace('.{}.__'.format(dcc_name), '.__')
            try:
                version_module_obj = utils.import_module(version_module_path)
            except Exception:
                version_module_obj = None
            if version_module_obj:
                try:
                    plugin_version = version_module_obj.get_version()
                except Exception as exc:
                    logger.warning(
                        'Impossible to retrieve version for Artella Plugin module: {} | {}'.format(module_path, exc))

        for member in utils.iterate_module_members(sub_module_obj, predicate=inspect.isclass):
            register_plugin(member[1], plugin_config, os.path.dirname(sub_module), plugin_version)

    if not _PLUGINS:
        logger.warning('No Artella plugins found to load!')
        return

    ordered_plugins_list = list()
    for plugin_id, plugin_dict in _PLUGINS.items():
        plugin_class = plugin_dict['class']
        plugin_index = plugin_class.INDEX or -1
        index = 0
        for i, plugin_item in enumerate(ordered_plugins_list):
            plugin_item_index = plugin_item.values()[0]['index']
            if plugin_index < plugin_item_index:
                index += 1
        ordered_plugins_list.insert(index, {plugin_id: {'index': plugin_index, 'dict': plugin_dict}})

    for plugin_item in ordered_plugins_list:
        plugin_id = plugin_item.keys()[0]
        plugin_dict = plugin_item.values()[0]['dict']
        plugin_class = plugin_dict['class']
        plugin_config_dict = plugin_dict.get('config', dict())
        try:
            plugin_inst = plugin_class(plugin_config_dict)
        except Exception as exc:
            logger.error('Impossible to instantiate Artella Plugin: "{}" | {}'.format(plugin_id, exc))
            continue
        _PLUGINS[plugin_id]['plugin_instance'] = plugin_inst


def register_plugin(class_obj, config_path, plugin_path, plugin_version=None, plugin_interface=None):
    """
    Registers an Artella plugin instance into the manager
    :param class_obj:
    :param config_path:
    :param plugin_path:
    :param plugin_version:
    :param plugin_interface:
    :return:
    :rtype: bool
    """

    if not config_path or not os.path.isfile(config_path):
        logger.warning(
            'Impossible to register Artella Plugin: {} because its config file does not exists: {}!'.format(
                class_obj, config_path))
        return False

    plugin_interface = plugin_interface or plugin.ArtellaPlugin

    if not issubclass(class_obj, plugin_interface):
        return True

    plugin_id = None
    if hasattr(class_obj, 'ID'):
        plugin_id = getattr(class_obj, 'ID')
    if not plugin_id:
        plugin_id = class_obj.__name__
    if plugin_id in _PLUGINS:
        logger.warning('Artella Plugin: "{}" already registered!'.format(plugin_id))
        return True

    try:
        plugin_config = utils.read_json(config_path)
    except Exception:
        logger.warning(
            'Artella Plugin "{}" configuration file "{}" has not a proper structure. Skipping plugin ...'.format(
                plugin_id, config_path))
        return False

    plugin_name = plugin_config.get('name', None)
    plugin_package = plugin_config.get('package', None)
    plugin_icon = plugin_config.get('icon', None)
    plugin_resources = plugin_config.get('resources', None)

    # Register DCC resources, both DCC specific plugin implementation
    # resources and generic implementation resources
    plugin_resources_paths = list()
    if plugin_resources and plugin_path:
        plugin_path_base = os.path.basename(plugin_path)
        if plugin_path_base == dcc.name():
            plugin_resources_paths.append(os.path.join(plugin_path, plugin_resources))
            plugin_resources_paths.append(os.path.join(os.path.dirname(plugin_path), plugin_resources))
        else:
            plugin_resources_paths.append(os.path.join(plugin_path, plugin_resources))
        for plugin_resources_path in plugin_resources_paths:
            if os.path.isdir(plugin_resources_path):
                dcc.register_dcc_resource_path(plugin_resources_path)
                icons_path = os.path.join(plugin_resources_path, 'icons')
                if os.path.isdir(icons_path):
                    dcc.register_dcc_resource_path(icons_path)

    _PLUGINS[plugin_id] = {
        'name': plugin_name,
        'package': plugin_package,
        'icon': plugin_icon,
        'class': class_obj,
        'config': plugin_config,
        'version': plugin_version,
        'resource_paths': plugin_resources_paths
    }

    logger.info('Artella Plugin: "{}" registered successfully!'.format(plugin_id))

    return True


def shutdown(dev=False):
    """
    Unloads all current loaded Artella plugins
    """

    if not _PLUGINS:
        return

    for plugin_id, plugin_dict in _PLUGINS.items():
        plugin_inst = plugin_dict.get('plugin_instance', None)
        if not plugin_inst:
            continue
        if dev:
            plugin_inst.cleanup()
        else:
            dcc.execute_deferred(plugin_inst.cleanup)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC abstract UI implementation
"""

from __future__ import print_function, division, absolute_import

from artella.core.dcc import reroute
from artella.core.utils import abstract





#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella Splash widget
"""

from __future__ import print_function, division, absolute_import

import os
import logging

from artella import dcc
from artella.core.dcc import dialog
from artella.core import utils, qtutils, resource
from artella.widgets import stack

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets, QtGui

logger = logging.getLogger('artella')


if not qtutils.QT_AVAILABLE:
    class SplashScreen(object):
        def __init__(self, *args, **kwargs):
            pass

    class ProgressCricle(object):
        def __init__(self, *args, **kwargs):
            pass
else:
    class SplashScreen(QtWidgets.QSplashScreen, object):
        def __init__(self, *args, **kwargs):
            super(SplashScreen, self).__init__(*args, **kwargs)

        def mousePressEvent(self, event):
            pass

    class ProgressCricle(QtWidgets.QProgressBar, object):
        def __init__(self, parent=None, **kwargs):
            super(ProgressCricle, self).__init__(parent)

            self._infinite = False
            self._timer = QtCore.QTimer(self)
            self._timer.timeout.connect(self._on_increase_value)

            self._main_layout = QtWidgets.QHBoxLayout()
            self._default_label = QtWidgets.QLabel()
            self._default_label.setAlignment(QtCore.Qt.AlignCenter)
            self._main_layout.addWidget(self._default_label)
            self.setLayout(self._main_layout)
            self._color = QtGui.QColor(221, 235, 230)
            self._width = kwargs.get('width', 140)

            self.setTextDirection(self.Direction.BottomToTop)

            self._start_angle = 90 * 16
            self._max_delta_angle = 360 * 16
            self._height_factor = 1.0
            self._width_factor = 1.0

            self.setFixedSize(QtCore.QSize(self._width * self._width_factor, self._width * self._height_factor))

        @property
        def infinite(self):
            return self._infinite

        def set_widget(self, widget):
            self.setTextVisible(False)
            self._main_layout.addWidget(widget)

        def set_infinite(self, flag):
            self._timer.stop()
            if flag:
                self._timer.start(15)
            self._infinite = flag

        def paintEvent(self, event):
            if self.text() != self._default_label.text():
                self._default_label.setText(self.text())
            if self.isTextVisible() != self._default_label.isVisible():
                self._default_label.setVisible(self.isTextVisible())

            percent = utils.get_percent(self.value(), self.minimum(), self.maximum())
            total_width = self._width
            pen_width = int(3 * total_width / 50.0)
            radius = total_width - pen_width - 1

            painter = QtGui.QPainter(self)
            painter.setRenderHints(QtGui.QPainter.Antialiasing)

            # draw background circle
            pen_background = QtGui.QPen()
            pen_background.setWidth(pen_width)
            pen_background.setColor(QtGui.QColor(80, 120, 110))
            pen_background.setCapStyle(QtCore.Qt.RoundCap)
            painter.setPen(pen_background)
            painter.drawArc(pen_width / 2.0 + 1,
                            pen_width / 2.0 + 1,
                            radius,
                            radius,
                            self._start_angle,
                            -self._max_delta_angle)

            # draw foreground circle
            pen_foreground = QtGui.QPen()
            pen_foreground.setWidth(pen_width)
            pen_foreground.setColor(self._color)
            pen_foreground.setCapStyle(QtCore.Qt.RoundCap)
            painter.setPen(pen_foreground)
            painter.drawArc(
                pen_width / 2.0 + 1, pen_width / 2.0 + 1, radius, radius,
                self._start_angle, -percent * 0.01 * self._max_delta_angle)
            painter.end()

        def _on_increase_value(self):
            new_value = self.value() + 1
            if new_value >= self.maximum():
                new_value = 0
            self.setValue(new_value)


class SplashDialog(dialog.Dialog(), object):
    def __init__(self, parent=None, **kwargs):
        super(SplashDialog, self).__init__(parent, use_artella_header=False, **kwargs)

    def get_main_layout(self):
        main_layout = QtWidgets.QVBoxLayout()
        main_layout.setContentsMargins(10, 10, 10, 10)
        main_layout.setSpacing(5)

        return main_layout

    def setup_ui(self):
        super(SplashDialog, self).setup_ui()

        self.setWindowFlags(QtCore.Qt.Window | QtCore.Qt.FramelessWindowHint | QtCore.Qt.WA_DeleteOnClose)
        self.setAttribute(QtCore.Qt.WA_TranslucentBackground)

        splash_pixmap = resource.pixmap('artella_splash')
        splash = SplashScreen(splash_pixmap)
        splash.setMask(splash_pixmap.mask())
        self._splash_layout = QtWidgets.QVBoxLayout()
        self._splash_layout.setAlignment(QtCore.Qt.AlignBottom)
        splash.setLayout(self._splash_layout)
        self.main_layout.addWidget(splash)

        size_width = splash_pixmap.size().width() + 20
        size_height = splash_pixmap.size().height() + 20
        self.setFixedSize(QtCore.QSize(size_width, size_height))

        shadow_effect = QtWidgets.QGraphicsDropShadowEffect(self)
        shadow_effect.setBlurRadius(qtutils.dpi_scale(15))
        shadow_effect.setColor(QtGui.QColor(0, 0, 0, 150))
        shadow_effect.setOffset(qtutils.dpi_scale(0))
        self.setGraphicsEffect(shadow_effect)


class InfoSplashDialog(SplashDialog, object):
    def __init__(self, parent=None, **kwargs):
        super(InfoSplashDialog, self).__init__(parent, **kwargs)

    def setup_ui(self):
        super(InfoSplashDialog, self).setup_ui()

        self._progress_text = QtWidgets.QLabel('Wait please ...')
        progress_text_layout = QtWidgets.QHBoxLayout()
        progress_text_layout.addStretch()
        progress_text_layout.addWidget(self._progress_text)
        progress_text_layout.addStretch()

        self._splash_layout.addLayout(progress_text_layout)

    def set_text(self, text):
        self._progress_text.setText(text)


class ProgressSplashDialog(SplashDialog, object):
    def __init__(self, parent=None, **kwargs):

        self._is_cancelled = False

        super(ProgressSplashDialog, self).__init__(parent, **kwargs)

    def keyPressEvent(self, event):
        pass

        # if event.key() == QtCore.Qt.Key_Escape:
        #     self._is_cancelled = True
        # super(ProgressSplashDialog, self).keyPressEvent(event)

    def setup_ui(self):
        super(ProgressSplashDialog, self).setup_ui()

        self._stack = stack.SlidingOpacityStackedWidget(parent=self)
        self._splash_layout.addStretch()
        self._splash_layout.addWidget(self._stack)

        progress_widget = QtWidgets.QWidget(parent=self)
        progress_layout = QtWidgets.QVBoxLayout()
        progress_widget.setLayout(progress_layout)
        self._progress = ProgressCricle()
        progress_lyt = QtWidgets.QHBoxLayout()
        progress_lyt.addStretch()
        progress_lyt.addWidget(self._progress)
        progress_lyt.addStretch()
        self._progress_text = QtWidgets.QLabel('Wait please ...')
        progress_txt_lyt = QtWidgets.QHBoxLayout()
        progress_txt_lyt.addStretch()
        progress_txt_lyt.addWidget(self._progress_text)
        progress_txt_lyt.addStretch()
        progress_layout.addStretch()
        progress_layout.addLayout(progress_lyt)
        progress_layout.addLayout(progress_txt_lyt)

        self._stack.addWidget(progress_widget)

    def set_progress_text(self, text):
        self._progress.setFormat(text)

    def set_infinite(self, flag):
        self._progress.set_infinite(flag)
        if flag:
            self.set_progress_text('Please wait ...')

    def start(self, reset=True, infinite=False):

        if dcc.is_batch():
            self._log_progress()
            return

        if reset:
            self._progress.setValue(0)
            self._progress_text.setText('')

        self.set_infinite(infinite)

        self.exec_()

    def end(self):
        self._log_progress()
        self._progress.set_infinite(False)

        if dcc.is_batch():
            return

        self.fade_close()

    def get_min_progress_value(self):
        return self._progress.minimum()

    def get_max_progress_value(self):
        return self._progress.maximum()

    def set_min_progress_value(self, min_value):
        self._progress.setMinimum(min_value)

    def set_max_progress_value(self, max_value):
        self._progress.setMaximum(max_value)

    def get_progress_value(self):
        return self._progress.value()

    def set_progress_value(self, value, status=''):
        if not self._progress.infinite:
            self._progress.setValue(value)
        self._progress_text.setText(str(status))

    def is_cancelled(self):
        return self._is_cancelled

    def _log_progress(self):
        """
         Internal function that logs current progress into DCC output window
         """

        logger.debug('{} - {}'.format(self._progress_text.text(), self._progress.value()))


class DownloadSplashDialog(ProgressSplashDialog, object):
    def __init__(self, downloader, parent=None, **kwargs):
        super(DownloadSplashDialog, self).__init__(parent, **kwargs)

        self._downloader = downloader
        self._file_items = dict()

    def setup_ui(self):
        super(DownloadSplashDialog, self).setup_ui()

        main_widget = QtWidgets.QWidget()
        main_layout = QtWidgets.QVBoxLayout()
        main_widget.setLayout(main_layout)
        self._stack.addWidget(main_widget)

        download_widget = QtWidgets.QWidget()
        download_layout = QtWidgets.QVBoxLayout()
        download_widget.setLayout(download_layout)
        content_area = QtWidgets.QScrollArea(parent=self)
        content_area.setMinimumHeight(qtutils.dpi_scale(150))
        content_area.setWidgetResizable(True)
        content_area.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        # content_area.setSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        content_area.setWidget(download_widget)
        content_area.setStyleSheet('background-color: transparent; border: none;')
        self._download_layout = QtWidgets.QVBoxLayout()
        download_layout.addLayout(self._download_layout)

        main_layout.addStretch()
        main_layout.addWidget(content_area)

    def download(self, file_paths):
        if dcc.is_batch():
            self._log_progress()
            return

        # qtutils.clear_layout(self._download_layout)
        # self._file_items.clear()
        # for file_path in file_paths:
        #     file_path = utils.clean_path(file_path)
        #     new_download_item = DownloadItemWidget(file_path, parent=self)
        #     new_download_item.setVisible(False)
        #     self._file_items[file_path] = new_download_item
        #     self._download_layout.addWidget(new_download_item)

        self.exec_()

    def update_download(self, file_path, status, progress):

        pass

        # self._stack.setCurrentIndex(1)
        #
        # if not file_path or not self._file_items:
        #     return
        # file_path = utils.clean_path(file_path)
        # if file_path not in self._file_items:
        #     return
        #
        # download_item = self._file_items[file_path]
        # download_item.set_status(status, progress)
        # download_item.setVisible(True)


if qtutils.QT_AVAILABLE:
    class DownloadItemWidget(QtWidgets.QWidget):
        def __init__(self, file_path, parent=None):
            super(DownloadItemWidget, self).__init__(parent)

            download_layout = QtWidgets.QHBoxLayout()
            self.setLayout(download_layout)

            self._path_label = QtWidgets.QLabel(os.path.basename(file_path))
            self._progress_text = QtWidgets.QLabel('Waiting ...')
            self._progress = ProgressCricle(width=qtutils.dpi_scale(35))
            # self._progress.setTextVisible(False)
            download_layout.addWidget(self._progress)
            download_layout.addWidget(self._path_label)
            download_layout.addStretch()
            download_layout.addWidget(self._progress_text)
            download_layout.addStretch()

            self._progress_text.setVisible(False)

        def set_status(self, status, progress=None):
            status = status or ''
            self._progress_text.setText(str(status))
            if progress is not None:
                self._progress.setValue(progress)
